/**
 * 扩展点类型，默认值为 array，可选值有以下几种：
 * array: 数组形式，支持多值，所有扩展配置都有效；
 * mixin: 混合形式，由多个扩展配置混合形成最后有效的配置；
 * single: 单一对象，只有最后的扩展配置有效
 */
export var Type = Object.freeze({
    Single: 'SINGLE',
    Mixin: 'MIXIN',
    Multiple: 'MULTIPLE'
});
/**
 * 默认的扩展点声明实现类
 */
var DefaultExtensionPoint = /** @class */ (function () {
    function DefaultExtensionPoint(point) {
        this.extensions = [];
        this.extension = {};
        this.preprocessors = [];
        this.processed = false;
        this.type = point.type;
        this.module = point.module;
    }
    /**
     * 添加扩展
     * @param extensions 扩展配置集合
     */
    DefaultExtensionPoint.prototype.addExtension = function () {
        var _this = this;
        var _a;
        var extensions = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            extensions[_i] = arguments[_i];
        }
        this.processed = false;
        (_a = this.extensions).push.apply(_a, extensions);
        switch (this.type) {
            case Type.Single:
                this.extension = extensions[extensions.length - 1];
                break;
            case Type.Mixin:
                extensions.forEach(function (item) { return Object.assign(_this.extension, item); });
                break;
            case Type.Multiple:
            default:
        }
    };
    /**
     * 获取扩展
     * @returns 当扩展点类型为 Multiple 时，返回扩展配置对象数组；
     *          当扩展点类型为 Single 时，返回最后加入的扩展配置对象；
     *          当扩展点类型为 Mixin 时，返回所有扩展配置对象混合后的结果
     */
    DefaultExtensionPoint.prototype.getExtension = function () {
        var _this = this;
        if (this.processed) {
            return this.extension;
        }
        else {
            var extension_1 = null;
            switch (this.type) {
                case Type.Single:
                case Type.Mixin:
                    extension_1 = this.extension;
                    break;
                case Type.Multiple:
                default:
                    extension_1 = this.extensions;
            }
            this.preprocessors.forEach(function (processor) {
                var result = processor.process(_this.extensions, _this);
                if (result !== null) {
                    extension_1 = result;
                }
            });
            this.extension = extension_1;
            this.processed = true;
            return extension_1;
        }
    };
    /**
     * 获取全部扩展配置对象
     */
    DefaultExtensionPoint.prototype.getExtensions = function () {
        return this.extensions;
    };
    /**
     * 添加预处理器
     * @param proprocessors 预处理器集合
     */
    DefaultExtensionPoint.prototype.addPreprocessors = function () {
        var _a;
        var proprocessors = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            proprocessors[_i] = arguments[_i];
        }
        this.processed = false;
        (_a = this.preprocessors).push.apply(_a, proprocessors);
    };
    return DefaultExtensionPoint;
}());
export { DefaultExtensionPoint };
