import LogInfo from './LogInfo';
import ModulesLoader from './ModulesLoader';
import { DefaultExtensionPoint, Type } from './ExtensionPoint';
/**
 * 模块化应用实现类
 */
var Modular = /** @class */ (function () {
    /**
     * 构造函数
     * @param config 初始配置
     */
    function Modular(options) {
        this.inited = false;
        this.logs = []; // 记录处理过程中产生的日志信息
        this.extensionPoints = {};
        this.attributes = {};
        options = options || { modules: [] };
        this.strict = !!options.strict; // 严格模式，暂未使用，保留
        this.modules = options.modules || [];
        this.application = options.application || { name: 'Application' };
        this.init();
    }
    /**
     * 获取应用配置
     */
    Modular.prototype.getApplication = function () {
        return this.application;
    };
    /**
     * 获取指定名称的模块配置
     * @param name 模块名称
     */
    Modular.prototype.getModule = function (name) {
        return this.modules.find(function (module) { return module.name === name; });
    };
    /**
     * 获取全部模块配置
     */
    Modular.prototype.getModules = function () {
        return this.modules;
    };
    /**
     * 获取指定名称的有效扩展配置
     * @param name 扩展点名称
     */
    Modular.prototype.getExtension = function (name) {
        var point = this.getExtensionPoint(name);
        if (point !== null) {
            return point.getExtension();
        }
        return null;
    };
    /**
     * 获取全部有效的扩展配置
     */
    Modular.prototype.getExtensions = function (name) {
        var point = this.getExtensionPoint(name);
        if (point !== null) {
            return point.getExtensions();
        }
        return null;
    };
    /**
     * 获取指定名称的扩展点定义
     * @param name 扩展点名称
     */
    Modular.prototype.getExtensionPoint = function (name) {
        var point = this.extensionPoints[name];
        if (point === undefined) {
            return null;
        }
        return point;
    };
    /**
     * 获取全部有效的扩展点定义
     */
    Modular.prototype.getExtensionPoints = function () {
        return this.extensionPoints;
    };
    /**
     * 获取属性名集合
     */
    Modular.prototype.getAttributeNames = function () {
        return Object.keys(this.attributes);
    };
    /**
     * 设置属性
     * @param name 属性名
     * @param value 属性值
     */
    Modular.prototype.setAttribute = function (name, value) {
        this.attributes[name] = value;
    };
    /**
     * 获取属性值
     * @param name 属性名
     */
    Modular.prototype.getAttribute = function (name) {
        return this.attributes[name];
    };
    /**
     * 启动模块化应用
     */
    Modular.prototype.start = function () {
        var _this = this;
        this.modules.forEach(function (module) {
            if (module.activator && module.activator.start) {
                module.activator.start(_this, module);
            }
        });
    };
    /**
     * 获取日志记录
     */
    Modular.prototype.getLogs = function () {
        return Object.freeze(this.logs);
    };
    /**
     * 记录日志
     * @param info 日志信息对象
     */
    Modular.prototype.log = function (info) {
        this.logs.push(info);
        // tslint:disable-next-line:no-console
        // console.log(info.toString())
    };
    /**
     * 创建模块名称映射集合
     * @param modules 模块配置对象集合
     */
    Modular.prototype.createNameMapping = function (modules) {
        var _this = this;
        var nameMapping = {};
        modules.forEach(function (module) {
            if (module.name === undefined || module.name === '') {
                _this.log(new LogInfo('E01', 'error', { m: module }));
                return;
            }
            var name = module.name;
            if (nameMapping[name]) {
                _this.log(new LogInfo('E02', 'error', { m1: nameMapping[name], m2: module }));
                return;
            }
            nameMapping[name] = module;
        });
        return nameMapping;
    };
    /**
     * 初始化
     */
    Modular.prototype.init = function () {
        var _this = this;
        var _a;
        if (this.inited) {
            this.log(new LogInfo('E00', 'error'));
            return;
        }
        var app = this.application;
        var modules = this.modules;
        // 建立 name 查询索引
        var nameMapping = this.createNameMapping(modules);
        // 解析依赖，模块排序
        var modulesLoader = new ModulesLoader();
        // TODO 处理优先加载模块
        // modulesLoader.add(nameMapping['modular-core'])
        modules.forEach(function (module) {
            _this.loadDepens(module, modulesLoader, nameMapping);
        });
        // 处理应用配置
        this.loadDepens(app, modulesLoader, nameMapping);
        modules = modulesLoader.getModules();
        // 组装扩展配置
        var points = {};
        var len = modules.length;
        for (var i = 0; i < len; i++) {
            var module = modules[i];
            if (module.extensionPoints) {
                var ps = module.extensionPoints;
                for (var name_1 in ps) {
                    if (points[name_1]) {
                        this.log(new LogInfo('E05', 'error', { m: module, ep: name_1 }));
                    }
                    else {
                        var point = new DefaultExtensionPoint(ps[name_1]);
                        point.module = module.name;
                        points[name_1] = point;
                    }
                }
            }
            if (module.preprocessors) {
                var processors = module.preprocessors;
                for (var name_2 in processors) {
                    if (points[name_2]) {
                        points[name_2].addPreprocessors(processors[name_2]);
                    }
                    else {
                        this.log(new LogInfo('E06', 'error', { m: module, ep: name_2 }));
                    }
                }
            }
            if (module.extensions) {
                var ext = module.extensions;
                for (var name_3 in ext) {
                    if (points[name_3]) {
                        if (points[name_3].type === Type.Multiple && Array.isArray(ext[name_3])) {
                            (_a = points[name_3]).addExtension.apply(_a, ext[name_3]);
                        }
                        else {
                            points[name_3].addExtension(ext[name_3]);
                        }
                    }
                    else {
                        this.log(new LogInfo('E06', 'error', { m: module, ep: name_3 }));
                    }
                }
            }
            modules[i] = Object.freeze(module);
        }
        this.application = Object.freeze(app); // 应用配置
        this.modules = Object.freeze(modules);
        this.extensionPoints = Object.freeze(points);
    };
    /**
     * 深度遍历加载 Modules
     * @param module 当前模块
     * @param modulesLoader 模块加载器
     * @param nameMapping 模块名称索引
     * @param cache 当前处理中的模块缓存
     */
    Modular.prototype.loadDepens = function (module, modulesLoader, nameMapping, cache) {
        if (cache === void 0) { cache = {}; }
        if (module.name === undefined || module.name === '') {
            return false;
        }
        if (modulesLoader.contains(module)) {
            return true;
        }
        if (module.dependencies && module.dependencies.length) {
            module.dependencies = Object.freeze(module.dependencies);
            var ds = module.dependencies;
            var dslen = ds.length;
            for (var i = 0; i < dslen; i++) {
                var d = ds[i];
                if (nameMapping[d]) {
                    if (cache[d]) {
                        // 正在处理中的cache包含依赖项，说明出现循环依赖的情况，跳过
                        return true;
                    }
                    cache[d] = nameMapping[d];
                    if (this.loadDepens(nameMapping[d], modulesLoader, nameMapping, cache)) {
                        delete cache[d];
                        // 依赖项加载成功
                        continue;
                    }
                    else {
                        this.log(new LogInfo('E03', 'error', { m1: module, m2: d }));
                        return false;
                    }
                }
                else {
                    this.log(new LogInfo('E04', 'error', { m1: module, m2: d }));
                    return false;
                }
            }
            // 依赖项全部加载成功
        }
        modulesLoader.add(module);
        return true;
    };
    return Modular;
}());
export default Modular;
